#coding:utf-8

##################################################
# 汽车自动驾驶模拟程序(可定时执行)
##################################################
import binascii
import datetime
import json
import math
import os
import random
import threading
import time
import traceback

from lib.protocol.report.EventReport_protocol import EventReport_protocol
from lib.protocol.report.GPSReport_protocol import GPSReport_protocol
from lib.protocol.report.LoginReport_protocol import LoginReport_protocol
from lib.protocol.report.OBDReport_CAN_protocol import OBDReport_CAN_protocol
from lib.protocol.report.VersionReport_protocol import VersionReport_protocol
from lib.protocol.report.response.Common_response import Common_response
from lib.protocol.report.response.Update_response import Update_response
from lib.socket.ClientSocket import ClientSocket
from lib.socket.service.ProtocolSimulaterDataService import ProtocolSimulaterDataService


class M500PressureServer():
    def __init__(self):
        self.host = "10.100.12.32"           #模拟器发送消息的主机地址
        self.port = 9008                     #模拟器发送消息的端口号
        self.socket = None
        self.timeout = 360                   #socket的超时时间
        self.sendDur = 5                     #设置默认多久发一条消息
        self.serviceStatus = 0               #服务状态，0表示未启动，1表示启动
        self.gpsLine = []                    #GPS 轨迹
        self.gpsLineIndex = 0                #GPS 轨迹索引
        self.travelStatus = 0                #0，表示未行驶，1表示开始行驶
        self.carId = "M202003060520"         #车机号
        self.sn = 1                          #消息流水号
        self.oilExpend = 10                  #设置汽车每升能跑多少公里
        self.carSpeed = 60                   #设置车速度
        self.directAngle = 59                #设置默认方向角
        # 定义要发送的obd数据
        self.OBDdata = {"fireStatus":1,"ACCStatus":0,"engineSpeed":300,"speed":0,"meterMileage":6000,"totailMileage":600,"totalOilExpen":30,"totalRunTime":10}
        # 定义初始的obd数据，与上面的OBD数据保持一致，主要用于汽车行驶过程中数据变化量的计算
        self.OBDdataOri = {"fireStatus": 1, "ACCStatus": 0, "engineSpeed": 300, "speed": 0, "meterMileage": 6000,"totailMileage": 600, "totalOilExpen": 30, "totalRunTime": 10}
        self.threadName = ""                 #模拟器进程名字
        self.durTime = 60                    #线程持续时间
        self.info = {}                       #用于存放运行线程的字典


    def setHost(self,data):
        self.host = data
    def setPort(self,data):
        self.port = data
    def setTimeout(self,data):
        self.timeout = data
    def setSendDur(self,data):
        self.sendDur = data
    def setCarId(self,data):
        self.carId = data
    def setOilExpend(self,data):
        self.oilExpend = data
    def setSpeed(self,data):
        self.carSpeed = data
    def setThreadName(self,data):
        self.threadName = data
    def setInfo(self,data):
        self.info = data
    def setDurTime(self,data):
        self.durTime = data

    ######################################################
    #启动socket连接
    ######################################################
    def connectService(self):
        cliSocket = ClientSocket(self.host, self.port)
        cliSocket.setTimeOut(self.timeout)
        cliSocket.connect()
        self.socket = cliSocket

    ######################################################
    #发送登录消息
    ######################################################
    def sendLoginMsg(self):
        loginObj = LoginReport_protocol(WATER_CODE=self.sn,DEV_ID=self.carId)
        loginMsg = loginObj.generateLoginMsg()
        self.sendMsg(loginMsg)
        self.sn = self.sn +1

    ######################################################
    #发送版本消息
    ######################################################
    def sendVersionMsg(self):
        versionObj = VersionReport_protocol(msgCount=1,WATER_CODE=self.sn,DEV_ID=self.carId)
        versionMsg = versionObj.generateVersionMsg()
        self.sendMsg(versionMsg)
        self.sn = self.sn + 1

    ######################################################
    # 汽车开始行驶的服务方法
    ######################################################
    def serviceTravel(self):
        startTime = self.startTime
        while self.serviceStatus == 1:
            time.sleep(self.sendDur)
            gpsMsg = ""
            OBDMsg = ""
            if self.travelStatus == 0:
                latitude = self.gpsLine[self.gpsLineIndex]["lat"]
                longitude = self.gpsLine[self.gpsLineIndex]["lng"]
                gpsMsg = self.genGPSMsg(latitude,longitude)
                return
            elif self.travelStatus == 1:
                if self.gpsLineIndex < len(self.gpsLine):
                    OBDMsg = self.genOBDMsg(self.OBDdata["fireStatus"],self.OBDdata["ACCStatus"],self.OBDdata["engineSpeed"], \
                                            self.OBDdata["speed"],self.OBDdata["meterMileage"],self.OBDdata["totailMileage"], \
                                            self.OBDdata["totalOilExpen"],self.OBDdata["totalRunTime"])
                    self.OBDdata["engineSpeed"] = 3000
                    self.OBDdata["speed"] = self.carSpeed
                    self.OBDdata["meterMileage"] = self.OBDdata["meterMileage"] + int(self.sendDur * (self.OBDdata["speed"] * 1000 / 3600))
                    self.OBDdata["totailMileage"] = self.OBDdata["totailMileage"] + int(self.sendDur * (self.OBDdata["speed"] * 1000  / 3600))
                    oilExpend = self.oilExpend
                    self.OBDdata["totalOilExpen"] = self.OBDdata["totalOilExpen"] + int((self.sendDur * (self.OBDdata["speed"] * 1000 / 3600)) * (1000 / (oilExpend *1000)))
                    self.OBDdata["totalRunTime"] = self.OBDdata["totalRunTime"] + self.sendDur
                    latitude = self.gpsLine[self.gpsLineIndex]["lat"]
                    longitude = self.gpsLine[self.gpsLineIndex]["lng"]
                    gpsMsg = self.genGPSMsg(latitude, longitude)
                    self.gpsLineIndex = self.gpsLineIndex + 1
                    info = self.getCurTime() + " 纬度：" + str(latitude).replace("\n","") + "      经度：" + str(longitude).replace("\n","")
                    info.replace("\n","")
                    # print(info)
                    # self.writeToFile("result.txt", info + "\n", 1)
                elif self.gpsLineIndex == len(self.gpsLine):
                    self.gpsLineIndex = self.gpsLineIndex - 1
                    self.stopTravelService()
                self.carDataObj.setTodayTotalMilleage(self.carData["curDayTravel"]["todayTotalMilleage"] + self.OBDdata["totailMileage"] - self.OBDdataOri["totailMileage"])
                self.carDataObj.setTheMilleage(self.carData["curDayTravel"]["theMilleage"] + self.OBDdata["totailMileage"] -self.OBDdataOri["totailMileage"])
                self.carDataObj.setTotalMilleage(self.carData["travelData"]["totalMilleage"] + self.OBDdata["totailMileage"] -self.OBDdataOri["totailMileage"])
                temp = self.OBDdata["totailMileage"]
                self.OBDdataOri["totailMileage"] = temp
                self.carDataObj.setTodayTodayTotalOil(self.carData["curDayTravel"]["todayTotalOil"] + self.OBDdata["totalOilExpen"] - self.OBDdataOri["totalOilExpen"])
                self.carDataObj.setTheOil(self.carData["curDayTravel"]["theOil"] + self.OBDdata["totalOilExpen"] - self.OBDdataOri["totalOilExpen"])
                self.carDataObj.setTotalOil(self.carData["travelData"]["totalOil"] + self.OBDdata["totalOilExpen"] - self.OBDdataOri["totalOilExpen"])
                self.OBDdataOri["totalOilExpen"] = self.OBDdata["totalOilExpen"]
                self.carDataObj.setTodayTodayTotalTime(self.carData["curDayTravel"]["todayTotalTime"] + self.OBDdata["totalRunTime"] - self.OBDdataOri["totalRunTime"])
                self.carDataObj.setTheTime(self.carData["curDayTravel"]["theTime"] + self.OBDdata["totalRunTime"] - self.OBDdataOri["totalRunTime"])
                self.carDataObj.setTotalTime(self.carData["travelData"]["totalTime"] + self.OBDdata["totalRunTime"] - self.OBDdataOri["totalRunTime"])
                self.OBDdataOri["totalRunTime"] = self.OBDdata["totalRunTime"]
            if self.serviceStatus == 1:
                if OBDMsg != "":
                    self.sendMsg(OBDMsg)
                    self.sn = self.sn + 1
                if gpsMsg != "":
                    time.sleep(0.1)
                    self.sendMsg(gpsMsg)
                    self.sn = self.sn + 1
                time.sleep(self.sendDur)
            endTime = int(time.time())
            if (endTime - startTime) > self.durTime:
                if self.threadName in self.info["threadsObj"]:
                    self.info["threadsObj"].pop(self.threadName)
                    self.info["sucessT"][self.threadName] = self
                self.stopTravelService()



    ######################################################
    #点火，发送点火事件
    ######################################################
    def fireOn(self):
        self.setGpsLine()
        if not os.path.exists("data/protocolTools/carData/" + self.carId + ".json"):
            psdsObj = ProtocolSimulaterDataService()
            data = psdsObj.genDataTemplate()
            psdsObj.writeToFile("data/protocolTools/carData/" + self.carId + ".json",data)
        #读取车机行驶数据
        with open("data/protocolTools/carData/" + self.carId + ".json", "r", encoding="utf-8") as fi:
            content = fi.read()
            # 加if的作用是防止报错，详见：https://blog.csdn.net/longzhinuhou/article/details/86634949
            if len(content) > 0:
                conJson = json.loads(content)
            else:
                conJson = {"time": {"dateTime": "2020-08-31 10:47:43", "date": "2020-08-31", "time": "10:47:43"}, "curDayTravel": {"todayTotalMilleage": 2304, "todayTotalOil": 144, "todayTotalTime": 144, "theMilleage": 2304, "theOil": 144, "theTime": 144}, "travelData": {"totalMilleage": 2304, "totalOil": 144, "totalTime": 144}}
            timeStamp = time.time()
            timeArray = time.localtime(timeStamp)
            dateTimeM = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
            dateM = time.strftime("%Y-%m-%d", timeArray)
            timeM = time.strftime("%H:%M:%S", timeArray)
            dataFile = self.carId + ".json"
            self.carDataObj = ProtocolSimulaterDataService("data/protocolTools/carData/", dataFile)
            self.carDataObj.setData(conJson)
            if dateM == conJson["time"]["date"]:
                self.OBDdata["totailMileage"] = conJson["travelData"]["totalMilleage"]
                self.OBDdataOri["totailMileage"] = conJson["travelData"]["totalMilleage"]
                self.OBDdata["totalOilExpen"] = conJson["travelData"]["totalOil"]
                self.OBDdataOri["totalOilExpen"] = conJson["travelData"]["totalOil"]
                self.OBDdata["totalRunTime"] = conJson["travelData"]["totalTime"]
                self.OBDdataOri["totalRunTime"] = conJson["travelData"]["totalTime"]
            else:                                                          #如果不是当天日期，则将日期设置为当天，并写入车辆数据文件
                conJson["curDayTravel"]["todayTotalMilleage"] = 0             # 今日行驶总里程
                conJson["curDayTravel"]["todayTotalOil"] = 0                  # 今日行驶总油耗
                conJson["curDayTravel"]["todayTotalTime"] = 0                 # 今日行驶总时间
                self.carDataObj.setTodayTotalMilleage(0)
                self.carDataObj.setTodayTodayTotalOil(0)
                self.carDataObj.setTodayTodayTotalTime(0)
                self.carDataObj.setDateTime2file(dateTimeM)
                self.carDataObj.setData2file(dateM)
                self.carDataObj.setTime2file(timeM)
                self.OBDdata["totailMileage"] = conJson["travelData"]["totalMilleage"]
                self.OBDdataOri["totailMileage"] = conJson["travelData"]["totalMilleage"]
                self.OBDdata["totalOilExpen"] = conJson["travelData"]["totalOil"]
                self.OBDdataOri["totalOilExpen"] = conJson["travelData"]["totalOil"]
                self.OBDdata["totalRunTime"] = conJson["travelData"]["totalTime"]
                self.OBDdataOri["totalRunTime"] = conJson["travelData"]["totalTime"]
            self.carData = conJson
        fireOnEventObj = EventReport_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        gpsData = self.genGPSData(self.gpsLine[0]["lat"], self.gpsLine[0]["lng"])
        fireOnEventObj.setGPSPkg(gpsData)
        fireOnEventObj.setEventType("0010")
        firOnEventMsg = fireOnEventObj.generateEventMsg()
        self.sendMsg(firOnEventMsg)
        self.sn = self.sn + 1
        time.sleep(0.1)
        gpsMsg = self.genGPSMsg(self.gpsLine[0]["lat"],self.gpsLine[0]["lng"])
        self.sendMsg(gpsMsg)
        self.sn = self.sn + 1
        time.sleep(0.1)
        OBDMsg = self.genOBDMsg(self.OBDdata["fireStatus"],1,self.OBDdata["engineSpeed"], \
                                            self.OBDdata["speed"],self.OBDdata["meterMileage"],self.OBDdata["totailMileage"], \
                                            self.OBDdata["totalOilExpen"],self.OBDdata["totalRunTime"])
        self.sendMsg(OBDMsg)
        self.sn = self.sn + 1

    ######################################################
    # 熄火，发送熄火事件
    ######################################################
    def fireOff(self):
        gpsLineIndex = self.gpsLineIndex
        if gpsLineIndex >= len(self.gpsLine):
            gpsLineIndex = gpsLineIndex - 1
        fireOffEventObj = EventReport_protocol(DEV_ID=self.carId)
        gpsData = self.genGPSData(self.gpsLine[gpsLineIndex]["lat"],self.gpsLine[gpsLineIndex]["lng"])
        fireOffEventObj.setGPSPkg(gpsData)
        fireOffEventObj.setEventType("0011")
        fireOffEventMsg = fireOffEventObj.generateEventMsg()
        self.sendMsg(fireOffEventMsg)
        time.sleep(0.1)
        gpsMsg = self.genGPSMsg(self.gpsLine[gpsLineIndex]["lat"], self.gpsLine[gpsLineIndex]["lng"])
        self.sendMsg(gpsMsg)
        self.sn = self.sn + 1

    #发送消息
    def sendMsg(self,msg):
        self.socket.setTimeOut(self.timeout)
        self.socket.send(msg)
    #接收消息
    def revMsg(self):
        self.socket.setTimeOut(self.timeout)
        return self.socket.receive()

    #设置GPS轨迹
    def setGpsLine(self):
        fileName = self.getRandomGpsLine()
        with open("data/protocolTools/GPSLines/" + fileName,"r",encoding="utf-8") as fi:
            content = fi.read()
            conJson = json.loads(content)
            self.gpsLine = conJson["GPSLine"]
            self.gpsLineIndex = 0

    #读取文件内容
    def getFileContent(self,file):
        with open(file, "r", encoding="utf-8") as fi:
            content = fi.read()
            return content

    #随机获取一条GPS 轨迹
    def getRandomGpsLine(self):
        files = os.listdir("data/protocolTools/GPSLines/")
        filesLen = len(files)
        whichF = random.randint(0,filesLen - 1)
        return files[whichF]

    #根据特定参数，生成GPS消息
    def genGPSMsg(self,latitude,longtitude):
        gpsObj = GPSReport_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        gpsObj.setLatitude(latitude)
        gpsObj.setLongitude(longtitude)
        gpsObj.setDirectionAngle(self.getDirAngle())
        gpsObj.setSpeed(self.carSpeed)
        gpsObj.setOBDSpeed(self.carSpeed)
        timeS = int(time.time()) - 8 * 3600
        timeArray = time.localtime(timeS)
        UTCTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        gpsObj.setUTCTime(UTCTime)
        gpsObj.setGPSTimestamp(timeS)
        msg = gpsObj.generateGpsMsg()
        return msg
    #根据特定参数，生成GPS消息体，不包含消息头
    def genGPSData(self,latitude,longtitude):
        gpsObj = GPSReport_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        gpsObj.setLatitude(latitude)
        gpsObj.setLongitude(longtitude)
        gpsObj.setDirectionAngle(self.getDirAngle())
        gpsObj.setSpeed(self.carSpeed)
        gpsObj.setOBDSpeed(self.carSpeed)
        timeS = int(time.time()) - 8 * 3600
        timeArray = time.localtime(timeS)
        UTCTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        gpsObj.setUTCTime(UTCTime)
        gpsObj.setGPSTimestamp(timeS)
        data = gpsObj.generateGpsData()
        return data

    # 根据特定参数，生成OBD CAN消息
    def genOBDMsg(self,fireStatus=1,ACCStatus=0,engineSpeed=300,speed=0,meterMileage=6000, \
                  totailMileage=600,totalOilExpend=30,totalRunTime=10):
        OBDObj = OBDReport_CAN_protocol(DEV_ID=self.carId,WATER_CODE=self.sn)
        timeS = int(time.time()) - 8 * 3600
        timeArray = time.localtime(timeS)
        UTCTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        OBDObj.setInfoTime(UTCTime)
        OBDObj.setFireStatus(fireStatus)
        OBDObj.setACCStatus(ACCStatus)
        OBDObj.setEngineSpeed(engineSpeed)                 # 设置发动机转速
        OBDObj.setSpeed(speed)                             # 设置车辆速度
        OBDObj.setMeterMileage(meterMileage)               # 设置仪表里程值
        OBDObj.setTotalMileage(totailMileage)              # 设置总里程值
        OBDObj.setTotalOilExpend(totalOilExpend)           # 设置总耗油量
        OBDObj.setTotalRunTime(totalRunTime)               # 设置车辆运行时间
        msg = OBDObj.generateOBDReportCANMsg()
        return msg


    ########################################################
    #启动行驶的服务
    ########################################################
    def startTravelService(self):
        self.serviceStatus = 1
        self.travelStatus = 1
        self.serviceTravel()

    ########################################################
    # 停止行驶的服务(同时会停止线程)
    ########################################################
    def stopTravelService(self):
        self.fireOff()
        self.serviceStatus = 0
        self.gpsLine = []
        self.gpsLineIndex = 0
        self.travelStatus = 0
        self.socket.close()

    ########################################################
    #接收消息方法
    ########################################################
    def serviceRev(self):
        self.serviceStatus = 2              #2代表只启动了接收消息的进程，1代表了接收和发送都启动了
        while self.serviceStatus != 0:
            d = self.revMsg()
            d = str(binascii.b2a_hex(d))[2:][:-1]

    ########################################################
    #启动接收消息服务
    ########################################################
    def startRevService(self):
        if self.socket != None:
            t2 = threading.Thread(target=self.serviceRev, args=())
            t2.start()

    def getCurTime(self):
        curTime = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        return "[" + curTime + "]"

    def writeToFile(self,path,data,type=1):         #：代表追加  0：代表覆盖
        if(type == 1):
            with open(path, "a", encoding='utf-8') as fi:
                fi.write(data)
        else:
            with open(path, "w", encoding='utf-8') as fi:
                fi.write(data)


    ###########################################################
    # 获取方向角
    ###########################################################
    def getDirAngle(self):
        dire = self.directAngle
        if self.gpsLineIndex == 0:
            return int(self.directAngle)
        lngCut = (float(self.gpsLine[self.gpsLineIndex]["lng"]) - float(self.gpsLine[self.gpsLineIndex - 1]["lng"])) * 1000000
        latCut = (float(self.gpsLine[self.gpsLineIndex]["lat"]) - float(self.gpsLine[self.gpsLineIndex - 1]["lat"])) * 1000000
        if latCut == 0:  # 除数维度不能为0
            latCut = 1
        if lngCut == 0 or latCut == 0:
            return int(self.directAngle)
        val = lngCut / latCut
        dire = math.atan2(1, val) * 180 / math.pi
        if lngCut > 0 and latCut > 0:
            dire = 90 - dire
        if lngCut < 0 and latCut > 0:
            dire = 270 + 180 - dire
        elif latCut < 0 and lngCut > 0:
            dire = 270 - dire
        elif lngCut < 0 and latCut < 0:
            dire = 180 + 90 - dire
        self.directAngle = dire
        return int(dire)

    ########################################################
    # 启动模拟程序服务
    ########################################################
    def startService(self):
        self.startTime = int(time.time())
        while True:
            endTime = int(time.time())
            if (endTime - self.startTime) > self.durTime:
                break
            try:
                self.sn = 1
                self.connectService()
                # self.startRevService()                              #启动接收消息服务
                self.sendLoginMsg()
                time.sleep(0.5)
                self.sendVersionMsg()
                time.sleep(0.5)
                self.fireOn()
                time.sleep(2)
                self.startTravelService()
                time.sleep(180)
            except BaseException as e:
                traceback.print_exc()
                self.info["threadsObj"].pop(self.threadName)
                self.info["failT"][self.threadName] = self

                timeS = int(time.time())
                timeArray = time.localtime(timeS)
                datetime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
                err = traceback.format_exc()
                self.writeToFile("data/log1.txt", "[" + datetime + "]", type=1)
                self.writeToFile("data/log1.txt", err, type=1)
                print(self.threadName + "线程出现异常\n")
                self.writeToFile("data/log1.txt",self.threadName + "线程出现异常\n", type=1)
            if (self.durTime - (endTime - self.startTime)) > 180:
                pass
            else:
                break
        if self.threadName in self.info["threadsObj"]:
            self.info["threadsObj"].pop(self.threadName)
            self.info["sucessT"][self.threadName] = self
        print(self.threadName + "线程结束\n")
        self.writeToFile("data/log1.txt",self.threadName + "线程结束\n", type=1)



